@@ -23,7 +23,7 @@
 
 #include <string.h>
 
-static GPtrArray *extra_attributes = NULL;
+GPtrArray *extra_attributes = NULL;
 
 enum {
   TEXT_CHANGED,
@@ -33,95 +33,43 @@ enum {
   LAST_SIGNAL
 };
 
-static const char boolean[] =
-  "false\0"
-  "true";
-static const guint8 boolean_offsets[] = {
-  0, 6
-};
+static const gchar *bool[] = {"false",
+                              "true"};
+static const gchar *style[] = {"normal",
+                               "oblique",
+                               "italic"};
+static const gchar *variant[] = {"normal",
+                                 "small_caps"};
+static const gchar *stretch[] = {"ultra_condensed",
+                                 "extra_condensed",
+                                 "condensed",
+                                 "semi_condensed",
+                                 "normal",
+                                 "semi_expanded",
+                                 "expanded",
+                                 "extra_expanded",
+                                 "ultra_expanded"};
+static const gchar *justification[] = {"left",
+                                       "right",
+                                       "center",
+                                       "fill"};
+static const gchar *direction[] = {"none",
+                                   "ltr",
+                                   "rtl"};
+static const gchar *wrap_mode[] = {"none",
+                                   "char",
+                                   "word"};
+static const gchar *underline[] = {"none",
+                                   "single",
+                                   "double",
+                                   "low"};
 
-static const char style[] =
-  "normal\0"
-  "oblique\0"
-  "italic";
-static const guint8 style_offsets[] = {
-  0, 7, 15
-};
-
-static const char variant[] =
-  "normal\0"
-  "small_caps";
-static const guint8 variant_offsets[] = {
-  0, 7
-};
-
-static const char stretch[] =
-  "ultra_condensed\0"
-  "extra_condensed\0"
-  "condensed\0"
-  "semi_condensed\0"
-  "normal\0"
-  "semi_expanded\0"
-  "expanded\0"
-  "extra_expanded\0"
-  "ultra_expanded";
-static const guint8 stretch_offsets[] = {
-  0, 16, 32, 42, 57, 64, 78, 87, 102
-};
-
-static const char justification[] =
-  "left\0"
-  "right\0"
-  "center\0"
-  "fill";
-static const guint8 justification_offsets[] = {
-  0, 5, 11, 18
-};
-
-static const char direction[] =
-  "none\0"
-  "ltr\0"
-  "rtl";
-static const guint8 direction_offsets[] = {
-  0, 5, 9
-};
-
-static const char wrap_mode[] =
-  "none\0"
-  "char\0"
-  "word\0"
-  "word_char";
-static const guint8 wrap_mode_offsets[] = {
-  0, 5, 10, 15
-};
-
-static const char underline[] =
-  "none\0"
-  "single\0"
-  "double\0"
-  "low";
-static const guint8 underline_offsets[] = {
-  0, 5, 12, 19
-};
-
-static void atk_text_base_init (AtkTextIface *class);
-
-static void atk_text_real_get_range_extents  (AtkText          *text,
-                                              gint             start_offset,
-                                              gint             end_offset,
-                                              AtkCoordType     coord_type,
-                                              AtkTextRectangle *rect);
-
-static AtkTextRange** atk_text_real_get_bounded_ranges (AtkText          *text,
-                                                        AtkTextRectangle *rect,
-                                                        AtkCoordType     coord_type,
-                                                        AtkTextClipType  x_clip_type,
-                                                        AtkTextClipType  y_clip_type);
+static void atk_text_base_init (gpointer *g_class);
 
 static guint atk_text_signals[LAST_SIGNAL] = { 0 };
 
 GType
-atk_text_get_type (void)
+atk_text_get_type ()
 {
   static GType type = 0;
 
@@ -144,7 +92,7 @@ atk_text_get_type (void)
 }
 
 static void
-atk_text_base_init (AtkTextIface *class)
+atk_text_base_init (gpointer *g_class)
 {
   static gboolean initialized = FALSE;
   
@@ -154,10 +102,7 @@ atk_text_base_init (AtkTextIface *class)
        * Note that text_changed signal supports details "insert", "delete", 
        * possibly "replace". 
        */
-     
-      class->get_range_extents = atk_text_real_get_range_extents; 
-      class->get_bounded_ranges = atk_text_real_get_bounded_ranges; 
-
+      
       atk_text_signals[TEXT_CHANGED] =
 	g_signal_new ("text_changed",
 		      ATK_TYPE_TEXT,
@@ -246,6 +191,9 @@ atk_text_get_character_at_offset (AtkText      *text,
 
   g_return_val_if_fail (ATK_IS_TEXT (text), (gunichar) 0);
 
+  if (offset < 0)
+    return (gunichar) 0;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_character_at_offset)
@@ -259,9 +207,8 @@ atk_text_get_character_at_offset (AtkText      *text,
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
- * @start_offset: the start offset of the returned string
- * @end_offset: the offset of the first character after the 
- *              returned substring
+ * @start_offset: the start offset of the returned string.
+ * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
  *
@@ -341,9 +288,8 @@ atk_text_get_text_after_offset (AtkText          *text,
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
- * @start_offset: the start offset of the returned string
- * @end_offset: the offset of the first character after the 
- *              returned substring
+ * @start_offset: the start offset of the returned string.
+ * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
  *
@@ -414,6 +360,9 @@ atk_text_get_text_at_offset (AtkText          *text,
   else
     real_end_offset = &local_end_offset;
 
+  if (offset < 0)
+    return NULL;
+
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_text_at_offset)
@@ -427,9 +376,8 @@ atk_text_get_text_at_offset (AtkText          *text,
  * @text: an #AtkText
  * @offset: position
  * @boundary_type: An #AtkTextBoundary
- * @start_offset: the start offset of the returned string
- * @end_offset: the offset of the first character after the 
- *              returned substring
+ * @start_offset: the start offset of the returned string.
+ * @end_offset: the end offset of the returned string.
  *
  * Gets the specified text.
  *
@@ -522,28 +470,28 @@ atk_text_get_caret_offset (AtkText *text)
 {
   AtkTextIface *iface;
 
-  g_return_val_if_fail (ATK_IS_TEXT (text), 0);
+  g_return_val_if_fail (ATK_IS_TEXT (text), -1);
 
   iface = ATK_TEXT_GET_IFACE (text);
 
   if (iface->get_caret_offset)
     return (*(iface->get_caret_offset)) (text);
   else
-    return 0;
+    return -1;
 }
 
 /**
  * atk_text_get_character_extents:
  * @text: an #AtkText
- * @offset: The offset of the text character for which bounding information is required.
- * @x: Pointer for the x cordinate of the bounding box
- * @y: Pointer for the y cordinate of the bounding box
- * @width: Pointer for the width of the bounding box
- * @height: Pointer for the height of the bounding box
+ * @offset: position
+ * @x: x-position of character
+ * @y: y-position of character
+ * @width: width of character
+ * @height: height of character
  * @coords: specify whether coordinates are relative to the screen or widget window 
  *
- * Get the bounding box containing the glyph representing the character at 
- *     a particular text offset. 
+ * Given an @offset, the @x, @y, @width, and @height values are filled
+ * appropriately. 
  **/
 void
 atk_text_get_character_extents (AtkText *text,
@@ -589,29 +537,20 @@ atk_text_get_character_extents (AtkText *text,
 
   if (iface->get_character_extents)
     (*(iface->get_character_extents)) (text, offset, real_x, real_y, real_width, real_height, coords);
-
-  if (*real_width <0)
-    {
-      *real_x = *real_x + *real_width;
-      *real_width *= -1;
-    }
 }
 
 /**
  *atk_text_get_run_attributes:
  *@text: an #AtkText
- *@offset: the offset at which to get the attributes, -1 means the offset of
- *the character to be inserted at the caret location.
+ *@offset: the offset at which to get the attributes
  *@start_offset: the address to put the start offset of the range
  *@end_offset: the address to put the end offset of the range
  *
  *Creates an #AtkAttributeSet which consists of the attributes explicitly
  *set at the position @offset in the text. @start_offset and @end_offset are
  *set to the start and end of the range around @offset where the attributes are
- *invariant. Note that @end_offset is the offset of the first character
- *after the range.  See the enum AtkTextAttribute for types of text 
- *attributes that can be returned. Note that other attributes may also be 
- *returned.
+ *invariant. See the enum AtkTextAttribute for types of text attributes that 
+ *can be returned. Note that other attributes may also be returned.
  *
  *Returns: an #AtkAttributeSet which contains the attributes explicitly set
  *at @offset. This #AtkAttributeSet should be freed by a call to
@@ -638,7 +577,7 @@ atk_text_get_run_attributes (AtkText          *text,
   else
     real_end_offset = &local_end_offset;
 
-  if (offset < -1)
+  if (offset < 0)
     return NULL;
 
   iface = ATK_TEXT_GET_IFACE (text);
@@ -766,8 +705,7 @@ atk_text_get_n_selections (AtkText *text)
  * of the text region is assigned the number 0, etc.  Note that adding,
  * moving or deleting a selected region can change the numbering.
  * @start_offset: passes back the start position of the selected region
- * @end_offset: passes back the end position of (e.g. offset immediately past) 
- * the selected region
+ * @end_offset: passes back the end position of the selected region
  *
  * Gets the text from the specified selection.
  *
@@ -809,7 +747,7 @@ atk_text_get_selection (AtkText *text,
  * atk_text_add_selection:
  * @text: an #AtkText
  * @start_offset: the start position of the selected region
- * @end_offset: the offset of the first character after the selected region.
+ * @end_offset: the end position of the selected region
  *
  * Adds a selection bounded by the specified offsets.
  *
@@ -870,8 +808,7 @@ atk_text_remove_selection (AtkText *text,
  * of the text region is assigned the number 0, etc.  Note that adding,
  * moving or deleting a selected region can change the numbering.
  * @start_offset: the new start position of the selection
- * @end_offset: the new end position of (e.g. offset immediately past) 
- * the selection
+ * @end_offset: the new end position of the selection
  *
  * Changes the start and end offset of the specified selection.
  *
@@ -927,76 +864,6 @@ atk_text_set_caret_offset (AtkText *text,
     }
 }
 
-/**
- * atk_text_get_range_extents:
- * @text: an #AtkText
- * @start_offset: The offset of the first text character for which boundary 
- *        information is required.
- * @end_offset: The offset of the text character after the last character 
- *        for which boundary information is required.
- * @coord_type: Specify whether coordinates are relative to the screen or widget window.
- * @rect: A pointer to a AtkTextRectangle which is filled in by this function.
- *
- * Get the bounding box for text within the specified range.
- *
- * Since: 1.3
- **/
-void
-atk_text_get_range_extents (AtkText          *text,
-                            gint             start_offset,
-                            gint             end_offset,
-                            AtkCoordType     coord_type,
-                            AtkTextRectangle *rect)
-{
-  AtkTextIface *iface;
-
-  g_return_if_fail (ATK_IS_TEXT (text));
-  g_return_if_fail (rect);
-
-  if (start_offset < 0 || start_offset >= end_offset)
-    return;
- 
-  iface = ATK_TEXT_GET_IFACE (text);
-
-  if (iface->get_range_extents)
-    (*(iface->get_range_extents)) (text, start_offset, end_offset, coord_type, rect);
-}
-
-/**
- * atk_text_get_bounded_ranges:
- * @text: an #AtkText
- * @rect: An AtkTextRectagle giving the dimensions of the bounding box.
- * @coord_type: Specify whether coordinates are relative to the screen or widget window.
- * @x_clip_type: Specify the horizontal clip type.
- * @y_clip_type: Specify the vertical clip type.
- *
- * Get the ranges of text in the specified bounding box.
- *
- * Since: 1.3
- *
- * Returns: Array of AtkTextRange. The last element of the array returned 
- *          by this function will be NULL.
- **/
-AtkTextRange**
-atk_text_get_bounded_ranges (AtkText          *text,
-                             AtkTextRectangle *rect,
-                             AtkCoordType      coord_type,
-                             AtkTextClipType   x_clip_type,
-                             AtkTextClipType   y_clip_type)
-{
-  AtkTextIface *iface;
-
-  g_return_val_if_fail (ATK_IS_TEXT (text), NULL);
-  g_return_val_if_fail (rect, NULL);
-
-  iface = ATK_TEXT_GET_IFACE (text);
-
-  if (iface->get_bounded_ranges)
-    return (*(iface->get_bounded_ranges)) (text, rect, coord_type, x_clip_type, y_clip_type);
-  else
-    return NULL;
-}
-
 /**
  * atk_attribute_set_free:
  * @attrib_set: The #AtkAttributeSet to free
@@ -1058,7 +925,7 @@ atk_text_attribute_get_name (AtkTextAttribute attr)
 {
   GTypeClass *type_class;
   GEnumValue *value;
-  const gchar *name = NULL;
+  gchar *name = NULL;
 
   type_class = g_type_class_ref (ATK_TYPE_TEXT_ATTRIBUTE);
   g_return_val_if_fail (G_IS_ENUM_CLASS (type_class), NULL);
@@ -1139,6 +1006,7 @@ atk_text_attribute_for_name (const gchar *name)
   return type;
 }
 
+
 /**
  * atk_text_attribute_get_value:
  * @attr: The #AtkTextAttribute for which a value is required
@@ -1161,213 +1029,30 @@ atk_text_attribute_get_value (AtkTextAttribute attr,
     case ATK_TEXT_ATTR_STRIKETHROUGH:
     case ATK_TEXT_ATTR_BG_STIPPLE:
     case ATK_TEXT_ATTR_FG_STIPPLE:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (boolean_offsets));
-      return boolean + boolean_offsets[index];
+      g_assert (index >= 0 && index < 2);
+      return bool[index];
     case ATK_TEXT_ATTR_UNDERLINE:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (underline_offsets));
-      return underline + underline_offsets[index];
+      g_assert (index >= 0 && index < 4);
+      return underline[index];
     case ATK_TEXT_ATTR_WRAP_MODE:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (wrap_mode_offsets));
-      return wrap_mode + wrap_mode_offsets[index];
+      g_assert (index >= 0 && index < 3);
+      return wrap_mode[index];
     case ATK_TEXT_ATTR_DIRECTION:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (direction_offsets));
-      return direction + direction_offsets[index];
+      g_assert (index >= 0 && index < 3);
+      return direction[index];
     case ATK_TEXT_ATTR_JUSTIFICATION:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (justification_offsets));
-      return justification + justification_offsets[index];
+      g_assert (index >= 0 && index < 3);
+      return justification[index];
     case ATK_TEXT_ATTR_STRETCH:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (stretch_offsets));
-      return stretch + stretch_offsets[index];
+      g_assert (index >= 0 && index < 9);
+      return stretch[index];
     case ATK_TEXT_ATTR_VARIANT:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (variant_offsets));
-      return variant + variant_offsets[index];
+      g_assert (index >= 0 && index < 2);
+      return variant[index];
     case ATK_TEXT_ATTR_STYLE:
-      g_assert (index >= 0 && index < G_N_ELEMENTS (style_offsets));
-      return style + style_offsets[index];
+      g_assert (index >= 0 && index < 3);
+      return style[index];
     default:
       return NULL;
    }
 }
-
-static void
-atk_text_rectangle_union (AtkTextRectangle *src1,
-                          AtkTextRectangle *src2,
-                          AtkTextRectangle *dest)
-{
-  gint dest_x, dest_y;
-
-  dest_x = MIN (src1->x, src2->x);
-  dest_y = MIN (src1->y, src2->y);
-  dest->width = MAX (src1->x + src1->width, src2->x + src2->width) - dest_x;
-  dest->height = MAX (src1->y + src1->height, src2->y + src2->height) - dest_y;
-  dest->x = dest_x;
-  dest->y = dest_y;
-}
-
-static gboolean
-atk_text_rectangle_contain (AtkTextRectangle *clip,
-                            AtkTextRectangle *bounds,
-                            AtkTextClipType  x_clip_type,
-                            AtkTextClipType  y_clip_type)
-{
-  gboolean x_min_ok, x_max_ok, y_min_ok, y_max_ok;
-
-  x_min_ok = (bounds->x >= clip->x) ||
-             ((bounds->x + bounds->width >= clip->x) &&
-              ((x_clip_type == ATK_TEXT_CLIP_NONE) ||
-               (x_clip_type == ATK_TEXT_CLIP_MAX)));
-
-  x_max_ok = (bounds->x + bounds->width <= clip->x + clip->width) ||
-             ((bounds->x <= clip->x + clip->width) &&
-              ((x_clip_type == ATK_TEXT_CLIP_NONE) ||
-               (x_clip_type == ATK_TEXT_CLIP_MIN)));
-
-  y_min_ok = (bounds->y >= clip->y) ||
-             ((bounds->y + bounds->height >= clip->y) &&
-              ((y_clip_type == ATK_TEXT_CLIP_NONE) ||
-               (y_clip_type == ATK_TEXT_CLIP_MAX)));
-
-  y_max_ok = (bounds->y + bounds->height <= clip->y + clip->height) ||
-             ((bounds->y <= clip->y + clip->height) &&
-              ((y_clip_type == ATK_TEXT_CLIP_NONE) ||
-               (y_clip_type == ATK_TEXT_CLIP_MIN)));
-
-  return (x_min_ok && x_max_ok && y_min_ok && y_max_ok);
-  
-}
-
-static void 
-atk_text_real_get_range_extents (AtkText           *text,
-                                 gint              start_offset,
-                                 gint              end_offset,
-                                 AtkCoordType      coord_type,
-                                 AtkTextRectangle  *rect)
-{
-  gint i;
-  AtkTextRectangle cbounds, bounds;
-
-  atk_text_get_character_extents (text, start_offset,
-                                  &bounds.x, &bounds.y,
-                                  &bounds.width, &bounds.height,
-                                  coord_type);
-
-  for (i = start_offset + 1; i < end_offset; i++)
-    {
-      atk_text_get_character_extents (text, i,
-                                      &cbounds.x, &cbounds.y, 
-                                      &cbounds.width, &cbounds.height, 
-                                      coord_type);
-      atk_text_rectangle_union (&bounds, &cbounds, &bounds);
-    }
-
-  rect->x = bounds.x;
-  rect->y = bounds.y;
-  rect->width = bounds.width;
-  rect->height = bounds.height;
-}
-
-static AtkTextRange**
-atk_text_real_get_bounded_ranges (AtkText          *text,
-                                  AtkTextRectangle *rect,
-                                  AtkCoordType     coord_type,
-                                  AtkTextClipType  x_clip_type,
-                                  AtkTextClipType  y_clip_type)
-{
-  gint bounds_min_offset, bounds_max_offset;
-  gint min_line_start, min_line_end;
-  gint max_line_start, max_line_end;
-  gchar *line;
-  gint curr_offset;
-  gint offset;
-  gint num_ranges = 0;
-  gint range_size = 1;
-  AtkTextRectangle cbounds;
-  AtkTextRange **range;
-
-  range = NULL;
-  bounds_min_offset = atk_text_get_offset_at_point (text, rect->x, rect->y, coord_type);
-  bounds_max_offset = atk_text_get_offset_at_point (text, rect->x + rect->width, rect->y + rect->height, coord_type);
-
-  if (bounds_min_offset == 0 &&
-      bounds_min_offset == bounds_max_offset)
-    return NULL;
-
-  line = atk_text_get_text_at_offset (text, bounds_min_offset, 
-                                      ATK_TEXT_BOUNDARY_LINE_START,
-                                      &min_line_start, &min_line_end);
-  g_free (line);
-  line = atk_text_get_text_at_offset (text, bounds_max_offset, 
-                                      ATK_TEXT_BOUNDARY_LINE_START,
-                                      &max_line_start, &max_line_end);
-  g_free (line);
-  bounds_min_offset = MIN (min_line_start, max_line_start);
-  bounds_max_offset = MAX (min_line_end, max_line_end);
-
-  curr_offset = bounds_min_offset;
-  while (curr_offset < bounds_max_offset)
-    {
-      offset = curr_offset;
-
-      while (curr_offset < bounds_max_offset)
-        {
-          atk_text_get_character_extents (text, curr_offset,
-                                          &cbounds.x, &cbounds.y,
-                                          &cbounds.width, &cbounds.height,
-                                          coord_type);
-          if (!atk_text_rectangle_contain (rect, &cbounds, x_clip_type, y_clip_type))
-	    break;
-          curr_offset++;
-        }
-      if (curr_offset > offset)
-        {
-          AtkTextRange *one_range = g_new (AtkTextRange, 1);
-
-          one_range->start_offset = offset;
-          one_range->end_offset = curr_offset;
-          one_range->content = atk_text_get_text (text, offset, curr_offset);
-          atk_text_get_range_extents (text, offset, curr_offset, coord_type, &one_range->bounds);
-
-          if (num_ranges >= range_size - 1)
-            {
-              range_size *= 2;
-              range = g_realloc (range, range_size * sizeof (gpointer));
-            }
-          range[num_ranges] = one_range;
-          num_ranges++; 
-        }   
-      curr_offset++;
-      if (range)
-        range[num_ranges] = NULL;
-    }
-  return range;
-}
-
-/**
- * atk_text_free_ranges:
- * @ranges: A pointer to an array of  #AtkTextRange which is to be freed.
- *
- * Frees the memory associated with an array of AtkTextRange. It is assumed
- * that the array was returned by the function atk_text_get_bounded_ranges
- * and is NULL terminated.
- *
- * Since: 1.3
- **/
-void
-atk_text_free_ranges (AtkTextRange **ranges)
-{
-  AtkTextRange **first = ranges;
-
-  if (ranges)
-    {
-      while (*ranges)
-        {
-          AtkTextRange *range;
-
-          range = *ranges;
-          *ranges++;
-          g_free (range->content);
-          g_free (range);
-        }
-      g_free (first);
-    }
-}
